@@ -1,10 +1,10 @@
-
 """
 Contact Form tests
 """
 import mock
 
 from django import test
+from django.core import mail
 from django.core.urlresolvers import reverse
 from django.template import loader, TemplateDoesNotExist
 
@@ -81,10 +81,11 @@ class BaseEmailFormMixinTests(test.TestCase):
         get_message_dict.return_value = {"to": ["user@example.com"]}
 
         form = forms.BaseEmailFormMixin()
-        form.send_email(mock_request)
+        result = form.send_email(mock_request)
 
         email_class.assert_called_once_with(**get_message_dict.return_value)
         email_class.return_value.send.assert_called_once_with(fail_silently=False)
+        self.assertEqual(email_class.return_value.send.return_value, result)
 
     @mock.patch("contact_form.forms.EmailMessage", autospec=True, mocksignature=True)
     @mock.patch("contact_form.forms.BaseEmailFormMixin.get_message_dict")
@@ -109,6 +110,27 @@ class BaseEmailFormMixinTests(test.TestCase):
             "subject": get_subject.return_value,
         }, message_dict)
 
+    @mock.patch("contact_form.forms.BaseEmailFormMixin.get_subject")
+    @mock.patch("contact_form.forms.BaseEmailFormMixin.get_message")
+    def test_get_message_dict_adds_headers_when_present(self, get_message, get_subject):
+        email_headers = {"Reply-To": "user@example.com"}
+
+        class HeadersForm(forms.BaseEmailFormMixin):
+
+            def get_email_headers(self):
+                return email_headers
+
+        form = HeadersForm()
+        message_dict = form.get_message_dict()
+
+        self.assertEqual({
+            "from_email": form.from_email,
+            "to": form.recipient_list,
+            "body": get_message.return_value,
+            "subject": get_subject.return_value,
+            "headers": email_headers,
+        }, message_dict)
+
 
 class ContactFormTests(test.TestCase):
 
@@ -132,6 +154,22 @@ class ContactFormTests(test.TestCase):
             template_exists = 0
         self.assertTrue(template_exists, "Email message template does not exist")
 
+    def test_sends_mail_with_headers(self):
+        class ReplyToForm(forms.ContactForm):
+            email = forms.forms.EmailField()
+
+            def get_email_headers(self):
+                return {'Reply-To': self.cleaned_data['email']}
+
+        mock_request = test.RequestFactory().get('/')
+        reply_to_email = u'user@example.com'  # the user's email
+
+        form = ReplyToForm(data={'email': reply_to_email})
+        message = form.send_email(mock_request)
+
+        reply_to_header_email = mail.outbox[0].extra_headers['Reply-To']
+        self.assertEqual(reply_to_email, reply_to_header_email)
+
 
 class ContactModelFormTests(test.TestCase):
 
